indexing
   title:       "DURATION"
   cluster:     "timedate [timedate]";
   project:     "The Universal Time & Date UTDC";
   copyright:   "SIG Computer, 1995";
   author:      "Alexei Simonov (alex@eiffel.ru)";
   approved_by: "Frieder Monninger";
   original:    "22,Sep,95"
   version:     "1.0"
   done_at:     "SIG Computer (info@eiffel.ru)";
   license:     "GNU Library GPL - see included readme file";
   extrnl_name: "duration.e";

class DURATION
   inherit
      SHORT_DURATION
         undefine
            infix "<",
            infix "+",
            infix "-",
            clear,
            plus,
            minus,
            ratio,
            divide_by_duration,
            multiply_by,
            divide_by
         redefine
            as_hours,
            as_minutes,
            as_seconds
         end;

      LONG_DURATION
         undefine
            infix "<",
            infix "+",
            infix "-",
            clear,
            plus,
            minus,
            ratio,
            divide_by_duration,
            multiply_by,
            divide_by
         redefine
            as_years,
            as_months,
            as_days
         end;

   creation
      clear,
      set,
      set_from_compressed

   feature { ANY }

      set( yr, mh, dy, hr, me : INTEGER; sd : REAL ) is
         require
            date_time_valid : sd <  tt.seconds_in_minute and then
                              sd >  - tt.seconds_in_minute and then
                              me <  tt.minutes_in_hour and then
                              me >  - tt.minutes_in_hour and then
                              hr <  tt.hours_in_day and then
                              hr >  - tt.hours_in_day and then
                              dy <= tt.days_in_month_d and then
                              dy >= - tt.days_in_month_d and then
                              mh <= tt.months_in_year and then
                              mh >= - tt.months_in_year
         do
            set_date( yr, mh, dy );
            set_time( hr, me, sd );
         end -- set

      clear is
         do
            milliseconds  := tt.zero;
            absolute_days := tt.zero;
         end -- clear

      as_seconds : REAL is
         do
            Result := as_seconds_years (tt.one);
         end -- as_seconds

      as_minutes : REAL is
         do
            Result := as_seconds_years (tt.seconds_in_minute);
         end -- as_minutes

      as_hours : REAL is
         do
            Result := as_seconds_years (tt.seconds_in_hour);
         end -- as_hours

      as_days : REAL is
         do
            Result := as_seconds_years (tt.seconds_in_day);
         end -- as_days

      as_months : REAL is
         do
            Result := as_seconds_years (tt.seconds_in_month);
         end -- as_months

      as_years : REAL is
         do
            Result := as_seconds_years (tt.seconds_in_year);
         end -- as_years

      infix "<" ( ts : like Current ) : BOOLEAN is
         do
            Result :=
               ( Current.absolute_days <= ts.absolute_days ) and then
               ( Current.milliseconds  <  ts.milliseconds  )
         end -- infix "<"

      infix "-",
      minus ( d: like Current ) :  like Current is
         do
            !!Result.set_from_compressed (
                     absolute_days - d.absolute_days,
                     milliseconds - d.milliseconds )
         end -- infix "-", minus

      infix "+",
      plus ( d: like Current) :  like Current is
         do
            !!Result.set_from_compressed (
                     absolute_days + d.absolute_days,
                     milliseconds + d.milliseconds )
         end -- infix "+", plus

      multiply_by ( n : INTEGER ) :  like Current is
         do
            !!Result.set_from_compressed (
                     absolute_days * n,
                     milliseconds * n )
         end -- multiply_by

      divide_by ( n: INTEGER ) :  like Current is
         require else
            non_zero_divider : n /= tt.zero
         local
            dy, ms: INTEGER
         do
            dy := absolute_days // n;
            ms := ( milliseconds + (absolute_days \\ n) *
            tt.milliseconds_in_day ) / n;
            !!Result.set_from_compressed (dy, ms);
         end -- divide_by


      ratio,
      divide_by_duration( d: like Current ) : REAL is
         require else
            Non_Zero_param : d.as_seconds /= 0
         do
            Result := Current.as_seconds / d.as_seconds;
         end -- ratio, divide_by_duration


      set_from_compressed (day, millisec: INTEGER) is
         local
            ms, dy, d: INTEGER;
         do
            ms := millisec;
            dy := day;
            -- normalize
            if ms >= tt.milliseconds_in_day or ms <= - tt.milliseconds_in_day then
               dy := dy + ms // tt.milliseconds_in_day;
               ms := ms \\ tt.milliseconds_in_day;
            end; -- if
            if dy > tt.zero and ms < tt.zero then
               dy := dy - tt.one_day;
               ms := tt.milliseconds_in_day + ms;
            elseif dy < tt.zero and ms > tt.zero then
               dy := dy + tt.one_day;
               ms := ms - tt.milliseconds_in_day;
            end; -- if

           absolute_days := dy
           milliseconds := ms

         end -- set_from_compressed

   feature { NONE }

      as_seconds_years (constant : INTEGER) : REAL is
         local
            ms, abs_day : REAL
         do
            ms := milliseconds;
            abs_day := absolute_days;
            Result :=
               ms / ( tt.milliseconds_in_second * constant ) +
               ( abs_day / constant ) * tt.seconds_in_day;
         end -- as_seconds_years

end -- class DURATION

