indexing
   title:       "Short_Duration";
   cluster:     "timedate [timedate]";
   project:     "The Universal Time & Date UTDC";
   copyright:   "SIG Computer, 1995";
   author:      "Alexei Simonov (alex@eiffel.ru)";
   approved_by: "Frieder Monninger";
   original:    22,Sep,95;
   version:     1.0;
   done_at:     "SIG Computer (info@eiffel.ru)";
   license:     "GNU Library GPL - see included readme file";
   extrnl_name: "shrt_drt.e";

class SHORT_DURATION

   inherit
      SIMPLE_DURATION
         redefine
            infix "<"
         end;

   creation
      clear,
      set_time,
      set_seconds,
      set_from_milliseconds

   feature

      milliseconds : INTEGER;

   feature -- Commands

      set_time ( hr, me : INTEGER; sd : REAL ) is
         require
            time_valid : sd < tt.seconds_in_minute and then
                         sd > - tt.seconds_in_minute and then
                         me < tt.minutes_in_hour and then
                         me > - tt.minutes_in_hour
         do
            milliseconds := real_to_int (
            (hr * tt.seconds_in_hour + me * tt.seconds_in_minute + sd ) *
             tt.milliseconds_in_second );
         end -- set_time

      set_seconds ( seconds : REAL ) is
         do
            milliseconds := real_to_int ( seconds * tt.milliseconds_in_second );
         end -- set_seconds

      set_from_milliseconds (n: INTEGER) is
         do
            milliseconds := n
         end -- set_from_milliseconds

      clear is
         do
            milliseconds := tt.zero;
         end -- clear

   feature -- Queries

      hour_part : INTEGER is
         do
            Result := milliseconds // tt.milliseconds_in_second // tt.seconds_in_hour;
         end -- hour_part

      minute_part : INTEGER is
         do
            Result :=
            ( milliseconds //
            tt.milliseconds_in_second - tt.seconds_in_hour * hour_part ) //
            tt.seconds_in_minute;
         end -- minute_part

      second_part : REAL is
         local
            ms : REAL
         do
            ms := milliseconds;
            Result :=
            ms / tt.milliseconds_in_second - tt.seconds_in_hour * hour_part -
            tt.seconds_in_minute * minute_part;
         end -- second_part

      as_seconds : REAL is
         local
            ms : REAL
         do
            ms := milliseconds;
            Result := milliseconds / tt.milliseconds_in_second;
         end -- as_seconds

      as_minutes : REAL is
         do
            Result := as_seconds / tt.seconds_in_minute;
         end -- as_minutes


      as_hours : REAL is
         do
            Result := as_seconds / tt.seconds_in_hour;
         end -- as_hours


      whole_seconds : INTEGER is
         do
            Result := real_to_int (as_seconds)
         end -- whole_seconds

      whole_minutes : INTEGER is
         do
            Result := real_to_int (as_minutes)
         end -- whole_minutes

      whole_hours : INTEGER is
         do
            Result := real_to_int (as_hours)
         end -- whole_hours

      infix "<" ( ts : like CURRENT ) : BOOLEAN is
         do
            Result := current.milliseconds < ts.milliseconds;
         end -- infix "<"

      infix "+",
      plus ( d: like CURRENT ) : like CURRENT is
         do
            !!Result.set_from_milliseconds (milliseconds + d.milliseconds)
         end -- infix "+", plus

      infix "-",
      minus ( d: like CURRENT ) : like CURRENT is
         do
            !!Result.set_from_milliseconds (milliseconds - d.milliseconds)
         end -- infix "-", minus

      multiply_by ( n: INTEGER ) : like CURRENT is
         do
            !!Result.set_from_milliseconds (milliseconds * n)
         end -- multiply_by

      divide_by ( n : INTEGER ) : like CURRENT is
         require else
            non_zero_divider : n /= tt.zero
         do
            !!Result.set_from_milliseconds (milliseconds // n)
         end -- divide_by

      ratio,
      divide_by_duration ( d : like CURRENT ) : REAL is
         require else
            Non_Zero_param : d.milliseconds /= 0
         local
            ms : REAL
         do
            ms := milliseconds;
            Result := ms / d.milliseconds ;
         end -- ratio, divide_by_duration

end -- class SHORT_DURATION
